@@ -31,6 +31,11 @@ class SvmLightCorpus(interfaces.CorpusABC):
         <info> .=. <string>
     
     The "qid" feature (used for SVMlight ranking), if present, is ignored.
+    
+    Although not mentioned in the specification above, SVMlight also expect its 
+    feature ids to be 1-based (counting starts at 1). We convert features to 0-base
+    internally by decrementing all ids when loading a SVMlight input file, and 
+    increment them again when saving as SVMlight.
     """
     
     def __init__(self, fname):
@@ -63,7 +68,7 @@ class SvmLightCorpus(interfaces.CorpusABC):
                 raise ValueError('invalid format at line no. %i in %s' %
                                  (lineNo, self.fname))
             target, fields = parts[0], [part.rsplit(':', 1) for part in parts[1:]]
-            doc = [(int(p1), float(p2)) for p1, p2 in fields if p1 != 'qid'] # ignore 'qid' features
+            doc = [(int(p1) - 1, float(p2)) for p1, p2 in fields if p1 != 'qid'] # ignore 'qid' features, convert 1-based feature ids to 0-based
             yield doc
     
 
@@ -77,7 +82,8 @@ class SvmLightCorpus(interfaces.CorpusABC):
         logging.info("converting corpus to SVMlight format: %s" % fname)
         fout = open(fname, 'w')
         for doc in corpus:
-            fout.write("%i %s\n" % (0, ' '.join("%i:%f" % p for p in doc)))
+            pairs = ' '.join("%i:%f" % (termId + 1, termVal) for termId, termVal  in doc) # +1 to convert 0-base to 1-base
+            fout.write("0 %s\n" % pairs) # target class is always 0
         fout.close()
 #endclass SvmLightCorpus
 
